""""

"""

def extension_metadata():
  return {
      "primary_extension": True,
      "priority": 100,
      "setups": {
          "CAS_Production": "dips://samtest.ihep.ac.cn:9135/Configuration/Server",
          "CAS_Production-cert": "dips://samtest.ihep.ac.cn:9135/Configuration/Server",
      },
      "default_setup": "CAS_Production",
  }




#!/usr/bin/env python

import socket, subprocess, time

hostname = socket.gethostname()

commands = """
wget http://dirac-code.ihep.ac.cn/cepc/test/cepc_test_job.tgz  
tar xvfz cepc_test_job.tgz
cd cepc_test_job
echo
echo Job Start
./simu.sh
echo
if [ $? -eq 0 ]; then
    echo Job Done.
else
    echo Job Failed.
fi
"""

start = time.time()
subp = subprocess.Popen( commands, shell = True, stdout = subprocess.PIPE, stderr = subprocess.PIPE )
stdout, stderr = subp.communicate()
runningTime = time.time() - start

print('Host Name :', hostname)
print('Running Time :', runningTime)
print('\n')
if stdout:
    print('==============================Standard Output==============================\n')
    print(stdout)
    print('\n')
if stderr:
    print('==============================Standard Error===============================\n')
    print(stderr)



#!/usr/bin/env python
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import DIRAC
from DIRAC import S_OK, S_ERROR

from DIRAC.Core.Base import Script

Script.setUsageMessage( """
List all the tasks

Usage:
   %s [option]
""" % Script.scriptName )

Script.registerSwitch( "f:", "field=",          "Fields to list, seperated by comma. (Owner,OwnerDN,OwnerGroup,CreationTime,Progress,JobGroup,Site)" )
Script.registerSwitch( "w",  "whole",           "List all the fields" )
Script.registerSwitch( "l:", "limit=",          "Task number limit" )
Script.registerSwitch( "e",  "expired",         "Include expired tasks" )
Script.registerSwitch( "r",  "removed",         "Include removed tasks (only for admin)" )
Script.registerSwitch( "u",  "users",           "Include other users' tasks (only for admin)" )
Script.registerSwitch( "a",  "all",             "Show all tasks" )

Script.parseCommandLine( ignoreErrors = False )
options = Script.getUnprocessedSwitches()
args = Script.getPositionalArgs()

from DIRAC.Core.Security.ProxyInfo                         import getProxyInfo
from IHEPDIRAC.WorkloadManagementSystem.Client.TaskClient   import TaskClient
taskClient = TaskClient()

fieldTitle = {
  'TaskID'      : 'TaskID',
  'TaskName'    : 'TaskName',
  'Status'      : 'Status',
  'Owner'       : 'Owner',
  'OwnerDN'     : 'OwnerDN',
  'OwnerGroup'  : 'OwnerGroup',
  'CreationTime': 'CreationTime',
  'UpdateTime'  : 'UpdateTime',
  'Progress'    : 'Progress(T/(D|F|R|W|O))',
  'Site'        : 'Site',
  'JobGroup'    : 'JobGroup',
}

def showTable(title, content):
  widths = []
  column = 0
  for t in title:
    width = len(t)
    for c in content:
      width = max(width, len(c[column]))
    widths.append(width+1)
    column += 1

  showTitle(title, widths)
  showLine(widths)
  showContent(content, widths)

def showTitle(title, widths):
  for t,w in zip(title, widths):
    format = '%%-%ds' % w
    print(format % t,)
  print('')

def showLine(widths):
  for w in widths:
    print('-'*w,)
  print('')

def showContent(content, widths):
  for con in content:
    for c,w in zip(con, widths):
      format = '%%-%ds' % w
      print(format % c,)
    print('')

def getTitle(fields):
  title = []
  for field in fields:
    title.append(fieldTitle[field])
  return title

def getTasks(fields, tasks):
  content = []
  for task in tasks:
    line = []
    for field in fields:
      if field == 'Progress':
        prog = task[field]
        progStr = '%s/(%s|%s|%s|%s|%s)' \
          % (prog.get('Total', 0), prog.get('Done', 0), prog.get('Failed', 0),
             prog.get('Running', 0), prog.get('Waiting', 0), prog.get('Deleted', 0))
        line.append(progStr)
      else:
        line.append(str(task[field]))
    content.append(line)
  return content

def main():
  basicFields = ['TaskID','TaskName','Status']

  fields = ['Owner','OwnerGroup','CreationTime','Progress']
  limit = -1
  showExpired = False
  showOtherUser = False
  showRemoved = False
  for option in options:
    (switch, val) = option
    if switch == 'f' or switch == 'field':
      fields = val.split(',')
    if switch == 'w' or switch == 'whole':
      fields = ['Owner','OwnerGroup','CreationTime','UpdateTime','Progress','JobGroup','Site']
    if switch == 'l' or switch == 'limit':
      limit = int(val)
    if switch == 'e' or switch == 'expired':
      showExpired = True
    if switch == 'r' or switch == 'removed':
      showRemoved = True
    if switch == 'u' or switch == 'user':
      showOtherUser = True
    if switch == 'a' or switch == 'all':
      showExpired = True
      showOtherUser = True
      showRemoved = True

  fields = basicFields + fields;

  condDict = {}
  condDict['Status'] = ['Init', 'Ready', 'Processing', 'Finished']
  if showExpired:
    condDict['Status'].append('Expired')
  if showRemoved:
    condDict['Status'].append('Removed')

  if not showOtherUser:
    result = getProxyInfo()
    if result['OK']:
      condDict['OwnerDN'] = result['Value']['identity']

  orderAttribute = 'TaskID:DESC'

  result = taskClient.listTask(condDict, limit, 0, orderAttribute)
  if not result['OK']:
    print('Get task list error: %s' % result['Message'])
    return

  # Task value
  taskList = list(result['Value'])
  taskList.reverse()

  title = getTitle(fields)
  content = getTasks(fields, taskList)
  showTable(title, content)

if __name__ == '__main__':
  main()



""" JUNOTest

  A test class to test the software for the vo juno.

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library
standard_library.install_aliases()
from builtins import *
from IHEPDIRAC.ResourceStatusSystem.SAM.SAMTest.CEBaseTest import CEBaseTest


__RCSID__ = '$Id: $'


class JUNOTest( CEBaseTest ):
  """
    JUNOTest is used to test whether the juno's software is fine to run jobs.
  """

  def __init__( self, args = None, apis = None ):
    super( JUNOTest, self ).__init__( args, apis )


  @staticmethod
  def _judge( log ):
    idx = log.find( 'SNiPER::Context Terminated Successfully' )
    print("++++++++++++++++++++++++correct or not:", idx)
    if idx != -1:
      return 'OK'
    else:
      return 'Bad'



#!/usr/bin/env python

import os
import sys

from DIRAC import S_OK, S_ERROR, gConfig, gLogger, exit
from DIRAC.Core.Base import Script
import DIRAC

from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit

@DIRACScript()
def main():
    usageMsg = '''Get squids for a site.
    {0} [option|cfgfile] site'''.format(Script.scriptName)
    Script.setUsageMessage(usageMsg)

    Script.parseCommandLine(ignoreErrors=False)

    args = Script.getPositionalArgs()
    switches = Script.getUnprocessedSwitches()

    if len(args) == 0:
        site = DIRAC.siteName() 
    else: 
        site = args[0]

    squidurl = ""

    for squid in gConfig.getValue( 'Resources/Squids/%s' % ( site ), [] ):
        squidurl = "http://" + squid + ":3128 "
        print("%s" % squidurl)



#!/usr/bin/env python
"""
Reset files in transformation with specified status

Usage:
 ihepdirac-transformation-reset-files [option|cfgfile] TransID1 [TransID2 ...]

Example: 
 $ihepdirac-transformation-reset-files -t Problematic,Assigned 123 456

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit
from DIRAC.Core.Base import Script

import sys

Script.registerSwitch('t:', 'status=', 'Files status to be reset (default to "Problematic")')
Script.parseCommandLine(ignoreErrors = False)

from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

@DIRACScript()
def main():
    args = Script.getPositionalArgs()

    if len(args) == 0:
        Script.showHelp()
        exit(1)


    status = ['Problematic']

    switches = Script.getUnprocessedSwitches()
    for switch in switches:
        if switch[0] == 't' or switch[0] == 'status':
            status = switch[1].split(',')
            status = [s.strip() for s in status]

    tc = TransformationClient()

    for t in args:
        res = tc.getTransformation(t)
        if not res['OK']:
            gLogger.error('Failed to get transformation information for %s: %s' % (t, res['Message']))
            continue

        selectDict = {'TransformationID': res['Value']['TransformationID']}
        if status:
            selectDict['Status'] = status
        res = tc.getTransformationFiles(condDict=selectDict)
        if not res['OK']:
            gLogger.error('Failed to get transformation files: %s' % res['Message'])
            continue
        if not res['Value']:
            gLogger.debug('No file found for transformation %s' % t)
            continue

        lfns = [f['LFN'] for f in res['Value']]

        gLogger.notice('Reset files for status: %s' % status)
        res = tc.setFileStatusForTransformation(t, 'Unused', lfns)
        if not res['OK']:
            gLogger.error('Failed to reset file status: %s' % res['Message'])
            continue
        if 'Failed' in res['Value']:
            gLogger.warn('Could not reset some files: ')
            for lfn, reason in res['Value']['Failed'].items():
               gLogger.warn('%s: %s' % (lfn, reason))

        gLogger.notice('Updated file statuses to "Unused" for %d file(s)' % len(lfns))

        result = tc.setTransformationParameter(t, 'Status', 'Flush')
        if not result['OK']:
            gLogger.error('Can not flush transformation: %s' % result['Message'])
            continue

if __name__ == "__main__":
    main()



#!/usr/bin/env python

import socket, subprocess, time

hostname = socket.gethostname()

commands = """
ls /cvmfs/juno.ihep.ac.cn
more /etc/redhat-release
export CMTCONFIG=amd64_linux26
source /cvmfs/juno.ihep.ac.cn/centos7_amd64_gcc830/Pre-Release/J20v2r0-Pre0/setup.sh
echo $LD_LIBRARY_PATH
ls /usr/include/
ls /cvmfs/juno.ihep.ac.cn/centos7_amd64_gcc830/contrib/compat/
source /cvmfs/juno.ihep.ac.cn/centos7_amd64_gcc830/contrib/compat/bashrc
python /cvmfs/juno.ihep.ac.cn/centos7_amd64_gcc830/Pre-Release/J20v2r0-Pre0/offline/Examples/Tutorial/share/tut_detsim.py gun
"""

start = time.time()
subp = subprocess.Popen( ['bash', '-c', commands], stdout = subprocess.PIPE, stderr = subprocess.PIPE )
stdout, stderr = subp.communicate()
runningTime = time.time() - start

print('Host Name :', hostname)
print('Running Time :', runningTime)
print('\n')
if stdout:
    print('==============================Standard Output==============================\n')
    print(stdout)
    print('\n')
if stderr:
    print('==============================Standard Error===============================\n')
    print(stderr)



#!/usr/bin/env python
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import json

import DIRAC
from DIRAC import S_OK, S_ERROR
from DIRAC.Core.Base import Script

Script.setUsageMessage( """
Delete task and all jobs in the task

Usage:
   %s [option] ... [TaskID] ...
""" % Script.scriptName )

Script.parseCommandLine( ignoreErrors = False )
options = Script.getUnprocessedSwitches()
args = Script.getPositionalArgs()

from IHEPDIRAC.WorkloadManagementSystem.Client.TaskClient   import TaskClient
taskClient = TaskClient()

def deleteTask(taskID):
  result = taskClient.deleteTask(taskID)
  if not result['OK']:
    print('Delete task error: %s' % result['Message'])
    return
  print('Task %s deleted' % taskID)

def main():
  if len(args) < 1:
    Script.showHelp()
    return

  for taskID in args:
    taskID = int(taskID)
    deleteTask(taskID)
    print('')

if __name__ == '__main__':
  main()



#! /usr/bin/env python
"""
Register SE files under specified directory to DFC. Files must be locally readable

Usage:
  ihepdirac_dms_register_dir [option|cfgfile] DFCDir LocalDir SE

Example:
  ihepdirac_dms_register_dir /juno/user/z/zhangxm/9743 /junofs/grid/user/z/zhangxm/9743 IHEP-JUNOEOS

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import os
import sys

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit

@DIRACScript()
def main():

    Script.registerSwitch( 'e', 'existCheck', 'Check if file exists')
    Script.registerSwitch( 'q:', 'querySkip=', 'Skip files in the meta query')
    Script.registerSwitch( 'b:', 'bufferSize=', 'Register buffer size, default to 100')
    Script.parseCommandLine(ignoreErrors = False)

    from DIRAC.Core.Utilities.Adler import fileAdler
    from DIRAC.Core.Utilities.File import makeGuid
    from DIRAC.DataManagementSystem.Client.DataManager import DataManager

    from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
    fcc = FileCatalogClient('DataManagement/FileCatalog')

    args = Script.getPositionalArgs()

    if len(args) != 3:
        Script.showHelp()
        exit(1)

    dfcDir = args[0]
    if (not args[1]) or args[1].endswith(os.sep):
        localDir = args[1]
    else:
        localDir = args[1] + os.sep
    toSE = args[2]

    lfnQuery = []
    existCheck = False
    bufferSize = 100
    switches = Script.getUnprocessedSwitches()
    for switch in switches:
        if switch[0] == 'q' or switch[0] == 'querySkip':
            result = fcc.findFilesByMetadata({'juno_transfer': switch[1]}, '/')
            if result['OK']:
                lfnQuery += result['Value']
        if switch[0] == 'e' or switch[0] == 'existCheck':
            existCheck = True
        if switch[0] == 'b' or switch[0] == 'bufferSize':
            bufferSize = int(switch[1])

    lfnQuery = set(lfnQuery)

    counter = 0

    dm = DataManager()
    fileTupleBuffer = []
    for root, dirs, files in os.walk(localDir):
        for f in files:
            counter += 1

            fullFn = os.path.join(root, f)
            if not fullFn.startswith(localDir):
                gLogger.error('%s does not start with %s' % (fullFn, localDir))
                continue
            lastPart = fullFn[len(localDir):]
            lfn = os.path.join(dfcDir, lastPart)

            if lfn in lfnQuery:
                if counter%1000 == 0:
                    gLogger.notice('Skip file in query counter: %s' % counter)
                continue

            if existCheck:
                result = fcc.isFile(lfn)
                if result['OK'] and lfn in result['Value']['Successful'] and result['Value']['Successful'][lfn]:
                    if counter%1000 == 0:
                        gLogger.notice('Skip file existed counter: %s' % counter)
                    continue

            size = os.path.getsize( fullFn )
            adler32 = fileAdler( fullFn )
            guid = makeGuid()
            fileTuple = ( lfn, fullFn, size, toSE, guid, adler32 )
            fileTupleBuffer.append(fileTuple)
            gLogger.debug('Register to lfn: %s' % lfn)
            gLogger.debug('fileTuple: %s' % (fileTuple,))

            if len(fileTupleBuffer) >= bufferSize:
                result = dm.registerFile( fileTupleBuffer )
                if not result['OK']:
                    gLogger.error('Can not register %s' % fullFn)
                    exit(1)
                del fileTupleBuffer[:]
                gLogger.notice('%s files registered' % counter)

    if fileTupleBuffer:
        result = dm.registerFile( fileTupleBuffer )
        if not result['OK']:
            gLogger.error('Can not register %s' % fullFn)
            exit(1)
        del fileTupleBuffer[:]

    gLogger.notice('Total %s files registered' % counter)

if __name__ == "__main__":
    main()



#! /usr/bin/env python
"""
Remove all the files and directories from SE and DFC under specified directory

Usage:
  ihepdirac-dms-rm-dir [option|cfgfile] DFCDir

Example:
  ihepdirac-dms-rm-dir /juno/user/z/zhangxm/9743
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import os

from DIRAC import exit as DIRACExit, gLogger
from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
counterFile = 0
counterDir = 0

def removeDir(d):
    global counterFile, counterDir
    fcc = FileCatalogClient('DataManagement/FileCatalog')
    dm = DataManager()
    result = fcc.listDirectory(d)
    if not result['OK']:
        gLogger.error('Failed to list directory %s: %s' %
                      (d, result['Message']))
        return

    gLogger.notice('Removing dir: %s' % d)

    if result['Value']['Successful'][d]['Files']:
        files = result['Value']['Successful'][d]['Files']
        fileNumber = len(files)
        gLogger.notice(
            'Removing {0} files from dir "{1}"'.format(fileNumber, d))
        counterFile += fileNumber
        dm.removeFile(files)

    if result['Value']['Successful'][d]['SubDirs']:
        for subdir in result['Value']['Successful'][d]['SubDirs']:
            removeDir(subdir)

    counterDir += 1
    fcc.removeDirectory(d)

@DIRACScript()
def main():
    Script.parseCommandLine()

    args = Script.getPositionalArgs()
    if len(args) != 1:
        Script.showHelp(exitCode=1)
    dfcDir = args[0]
    fcc = FileCatalogClient('DataManagement/FileCatalog')
    dm = DataManager()
    retVal = 0
    removeDir(dfcDir)

    gLogger.notice('%s directories and %s files deleted' %
                  (counterDir, counterFile))

    DIRACExit(retVal)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

""" Upload files to SE, register to DFC and set metadata
"""

__RCSID__ = "$Id$"

# generic imports
import os
import glob
import json

# DIRAC imports
import DIRAC
from DIRAC.Core.Base import Script
Script.parseCommandLine()

from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from DIRAC.DataManagementSystem.Client.DataManager import DataManager
from DIRAC.Core.Utilities.SiteSEMapping import getSEsForSite


def getSiteSE(SEname):
    sitename = DIRAC.siteName()
    DIRAC.gLogger.error('Sitename: %s' % (sitename))
    print("sitename", sitename)
    res = getSEsForSite(sitename)
    if not res['OK']:
        DIRAC.gLogger.error(res['Message'])
        return SEname
    if res['Value']:
        SEname = res['Value'][0]
    return SEname

####################################################


def addDataFiles(args):
    fcc = FileCatalogClient()
    dm = DataManager(['FileCatalog'])

    outputPath = args[0]
    outputPattern = args[1]
    outputSE = args[2]
    mode = args[3]

    if mode == 'closest':
        outputSE = getSiteSE(outputSE)

    DIRAC.gLogger.error('OutputSE: %s' % (outputSE))
    print("outputSE", outputSE)

    # Create path
    res = fcc.createDirectory(outputPath)
    if not res['OK']:
        return res

    # Upload data files
    all_files = glob.glob(outputPattern)

    # Check that data files exist
    if len(all_files) == 0:
        return DIRAC.S_ERROR('No data files found')

    for one_file in all_files:
        lfn = os.path.join(outputPath, one_file)
        msg = 'Try to upload local file: %s \nwith LFN: %s \nto %s' % (
            one_file, lfn, outputSE)
        DIRAC.gLogger.notice(msg)
        res = dm.putAndRegister(lfn, one_file, outputSE)
        # Check if failed
        if not res['OK']:
            DIRAC.gLogger.error('Failed to putAndRegister %s \nto %s \nwith message: %s' % (
                lfn, outputSE, res['Message']))
            return res
        elif lfn in res['Value']['Failed'].keys():
            DIRAC.gLogger.error(
                'Failed to putAndRegister %s to %s' % (lfn, outputSE))
            return res

    return DIRAC.S_OK()


####################################################
if __name__ == '__main__':
    args = Script.getPositionalArgs()
    try:
        res = addDataFiles(args)
        if not res['OK']:
            DIRAC.gLogger.error(res['Message'])
            DIRAC.exit(-1)
    except Exception:
        DIRAC.gLogger.exception()
        DIRAC.exit(-1)



#! /usr/bin/env python
"""
Check if all files under specified DFC directory have a replica in specified SEs (splitted by commas).

Usage:
  ihepdirac_dms_check_replicas [option|cfgfile] DFCDir SEs

Example:
  ihepdirac_dms_check_replicas /juno/user/z/zhangxm/9743 IHEP-JUNOEOS

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import os
import sys

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit

@DIRACScript()
def main():

    Script.registerSwitch('f', 'fix', 'fix the discrepancy by replicating files from other SEs.')
    Script.parseCommandLine(ignoreErrors=False)

    args = Script.getPositionalArgs()
    switches = Script.getUnprocessedSwitches()


    dfcDir = args[0]
    SEs = args[1]
    SE_list = SEs.split(',')

    fixDiscrepancy = False


    for switch in switches: 
        if switch == 'fix':
            fixDiscrepancy = True

    #log
    gLogger.notice('Checking with DFC folder: {0}\nFor SE(s): {1}'.format(dfcDir,SEs))
    if fixDiscrepancy:
        gLogger.notice('Would fix the discrepancy by replicating files from other SEs.')
    gLogger.notice("\n\n")

    from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
    fcc = FileCatalogClient('DataManagement/FileCatalog')

    from DIRAC.DataManagementSystem.Client.DataManager import DataManager
    dm = DataManager()

    #get all files in dfcDir
    gf_result = dm.getFilesFromDirectory(dfcDir)
    if not gf_result['OK']:
       gLogger.error('Failed to list directory %s: %s' %(dfcDir, gf_result['Message']))
       exit()
    files = gf_result['Value']
    fileNumber = len(files)
    gLogger.notice('There are {0} files in dir {1}'.format(fileNumber,dfcDir))
	
    #get replicas in the SE
    gr_result = dm.getReplicasFromDirectory(dfcDir)
    if not gr_result['OK']:
        gLogger.error('Failed to get replicas from directory %s: %s' %(dfcDir, gr_result['Message']))
        exit()

    replicas = {SE:[] for SE in SE_list}
    for LFN in gr_result['Value'].keys():
        repInfo = gr_result['Value'][LFN]
        for SE in SE_list:
            repInfo_in_SE = repInfo.get(SE)	#Status, PFNif repInfo_in_SE is not None:
            if repInfo_in_SE is not None:
                replicas[SE].append(LFN)

    for SE in SE_list:
        replicaNumber = len(replicas[SE])
        gLogger.notice('There are {0} replicas in SE {1}'.format(replicaNumber,SE))

    #compare
    for SE in SE_list:
        files_not_in_SE = [f for f in files if f not in replicas[SE]]
        nFilesNotInSE = len(files_not_in_SE)

        nReplicate = 0

        gLogger.notice("\nThere are {0} files not in SE {1}.".format(nFilesNotInSE,SE))
        for f in files_not_in_SE:
            gLogger.notice('%s'%f)


        if fixDiscrepancy:
           gLogger.notice(' ')
           for f in files_not_in_SE:
               availableReplicas = gr_result['Value'][f]
               gLogger.notice('Start replicating LFN %s'%f)
               result = dm.replicateAndRegister(f, SE)
               if not result['OK']:
                   gLogger.error('Failed to replicate LFN %s to target SE %s: %s' %(f, SE, result['Message']))
               else:
                   gLogger.notice('Success')
                   nReplicate += 1

        gLogger.notice('{0} files successfully replicated'.format(nReplicate))

if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Register files in EOS SE Directory to DFC.

Usage:
  ihepdirac_dms_register_EOS_files [option|cfgfile] DFCDir EosDir SE 

Example:
  $ ihepdirac_dms_register_EOS_files /juno/raw/test_register root://junoeos01.ihep.ac.cn:1094//eos/juno/dirac/juno/raw/test_register IHEP-JUNOEOS
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit

import sys
import os

from XRootD import client
from XRootD.client.flags import QueryCode

from DIRAC.Core.Utilities.Adler import fileAdler
from DIRAC.Core.Utilities.File import makeGuid
from DIRAC.DataManagementSystem.Client.DataManager import DataManager

from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
fcc = FileCatalogClient('DataManagement/FileCatalog')

@DIRACScript()
def main():
    Script.parseCommandLine(ignoreErrors = False)
    args = Script.getPositionalArgs()

    if len(args) != 3:
        Script.showHelp()
        exit(1)

    dfcRoot = args[0]
    eosRoot = args[1] 
    toSE = args[2]

    group = eosRoot.split("//")
    eosHead = group[0] + "//" + group[1] + "/" 
    eosPath = "/" + group[2]

    lfnQuery = []
    existCheck = False
    bufferSize = 100
    switches = Script.getUnprocessedSwitches()
    for switch in switches:
        if switch[0] == 'q' or switch[0] == 'querySkip':
            result = fcc.findFilesByMetadata({'juno_transfer': switch[1]}, '/')
            if result['OK']:
                lfnQuery += result['Value']
        if switch[0] == 'e' or switch[0] == 'existCheck':
            existCheck = True
        if switch[0] == 'b' or switch[0] == 'bufferSize':
            bufferSize = int(switch[1])

    lfnQuery = set(lfnQuery)

    counter = 0
    eosclient = client.FileSystem(eosRoot)

    dm = DataManager()
    fileTupleBuffer = []

    cmd = 'xrdfs %s ls %s -R' % (eosHead, eosPath)
    gLogger.debug('Commands to get files:%s' % (cmd))
    file_obj = os.popen(cmd).readlines()
    #gLogger.debug('file lists:%s' % (file_obj))
    for fullFn in file_obj:
        counter += 1
        fullFn=fullFn.strip('\n')
        gLogger.debug('fullFn:%s' % (fullFn))

        if not fullFn.startswith(eosPath):
            gLogger.error('%s does not start with %s' % (fullFn, localDir))
            continue 
        lastPart = fullFn[len(eosPath):]
        pfn = eosHead + fullFn
      
        lfn = dfcRoot + lastPart
        gLogger.debug('lfn, dfcRoot, lastPart:%s, %s, %s' % (lfn, dfcRoot, lastPart))

        if lfn in lfnQuery:
            gLogger.notice('File exists, skip: %s' % lfn)
            counter -= 1
            continue

        if existCheck:
            result = fcc.isFile(lfn)
            if result['OK'] and lfn in result['Value']['Successful'] and result['Value']['Successful'][lfn]:
                gLogger.notice('File exists, skip: %s' % lfn)
                counter -= 1
                continue

        status, response = eosclient.stat(fullFn)
        if status.ok:
            size = response.size
        else:
            gLogger.error('Error in getting size of %s!' % lfn)
            continue
        # 19 is a directory
        if response.flags == 19:
           gLogger.debug('%s is a directory' % lfn)
           counter -= 1
           continue

        status, response = eosclient.query(QueryCode.CHECKSUM, fullFn)
        if status.ok:
            gchecksum = response.split()
        else:
            gLogger.error('%s is not a file, or has problems to get checksum!')% lfn
            continue
        if size != 0:
            #adler32 = gchecksum[1].strip(b'\x00'.decode())
            adler32 = gchecksum[1].strip(b'\x00')
            guid = makeGuid()
            fileTuple = ( lfn, pfn, size, toSE, guid, adler32 )
            gLogger.debug('To be registered: %s %s %s %s %s %s' % (lfn,pfn,size,toSE,guid,adler32))
            fileTupleBuffer.append(fileTuple)
            gLogger.debug('Register lfn: %s' % lfn)
        else:
            counter -=1

        if len(fileTupleBuffer) >= bufferSize:
            result = dm.registerFile( fileTupleBuffer )
            if not result['OK']:
                gLogger.error('Can not register %s' % pfn)
                exit(1)
            del fileTupleBuffer[:]
            gLogger.notice('%s files registered' % counter)

    if fileTupleBuffer:
        result = dm.registerFile( fileTupleBuffer )
        if not result['OK']:
            gLogger.error('Can not register %s' % fullFn)
            exit(1)
        del fileTupleBuffer[:]

    gLogger.notice('Total %s files registered' % counter)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

import os
import sys

from DIRAC.Core.Base import Script
Script.initialize(ignoreErrors=True)

from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC.Interfaces.API.Job import Job

from DIRAC.WorkloadManagementSystem.Client.JobReport import JobReport

jobID = os.environ.get('DIRACJOBID', '0')

if not jobID:
    print('DIRAC job ID not found')
    sys.exit(1)


jobReport = JobReport(jobID, 'JUNO_JobScript')
result = jobReport.setApplicationStatus(', '.join(sys.argv[1:]))
if not result['OK']:
    print('Set application status error: %s' % result)



#!/usr/bin/env python

import socket, time

start = time.time()
hostname = socket.gethostname()
runningTime = time.time() - start

print('hello, ' + hostname + '\n')
print('Running Time :', runningTime)



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import DIRAC
from DIRAC import S_OK, S_ERROR

from DIRAC.Core.Base import Script

Script.setUsageMessage( """
Show task detailed information

Usage:
   %s [option] ... [TaskID] ...
""" % Script.scriptName )

Script.registerSwitch( "j",  "job",             "Show job numbers" )

Script.parseCommandLine( ignoreErrors = False )
options = Script.getUnprocessedSwitches()
args = Script.getPositionalArgs()

from IHEPDIRAC.WorkloadManagementSystem.Client.TaskClient   import TaskClient
taskClient = TaskClient()

from DIRAC.Core.DISET.RPCClient                      import RPCClient
jobmonClient = RPCClient('WorkloadManagement/JobMonitoring')

def showPairs(pairs):
  width = 0
  for pair in pairs:
    width = max(width, len(pair[0]))
  format = '%%-%ds : %%s' % width
  for k,v in pairs:
    print(format % (k, v))

def showTask(taskID):
  outFields = ['TaskID','TaskName','Status','Owner','OwnerDN','OwnerGroup','CreationTime','UpdateTime','JobGroup','Site','Progress','Info']
  result = taskClient.getTask(taskID)
  if not result['OK']:
    print('Get task error: %s' % result['Message'])
    return False
  task = result['Value']

  pairsDict = {'Task':[], 'Progress':[], 'Info':[]}
  for k in outFields:
    if k in ['Progress', 'Info']:
      for kp,vp in sorted(task[k].iteritems(), key=lambda d:d[0]):
        if type(vp) == type([]):
          vp = ', '.join(vp)
        pairsDict[k].append([kp, vp])
    else:
      pairsDict['Task'].append([k, task[k]])

  showPairs(pairsDict['Task'])
  print('\n== Progress ==')
  showPairs(pairsDict['Progress'])
  print('\n== Information ==')
  showPairs(pairsDict['Info'])

def showTaskJobs(taskID):
  result = taskClient.getTaskJobs(taskID)
  if not result['OK']:
    print('Get task jobs error: %s' % result['Message'])
    return
  jobIDs = result['Value']
  print('== Jobs ==')
  if not jobIDs:
    print('No jobs found')
    return

  for jobID in jobIDs:
    print(jobID,)
  print('')

  result = jobmonClient.getJobs( { 'JobID': jobIDs, 'Status': ['Failed'] } )
  if not result['OK']:
    print('Get task failed jobs error: %s' % result['Message'])
    return
  print('\n== Jobs (Failed) ==')
  for jobID in result['Value']:
    print(jobID,)

def showTaskHistories(taskID):
  result = taskClient.getTaskHistories(taskID)
  if not result['OK']:
    print('Get task histories error: %s' % result['Message'])
    return

  print('\n== Job Histories ==')
  for status, statusTime, description in result['Value']:
    print('%-16s %-24s %s' % (status, statusTime, description))

def main():
  if len(args) < 1:
    Script.showHelp()
    return

  showJobNumber = False
  for option in options:
    (switch, val) = option
    if switch == 'j' or switch == 'job':
      showJobNumber = True

  for taskID in args:
    print('='*80)
    taskID = int(taskID)

    showTask(taskID)
    print('')

    showTaskHistories(taskID)
    print('')

    if showJobNumber:
      showTaskJobs(taskID)
      print('')

if __name__ == '__main__':
  main()



#! /usr/bin/env python
"""
Add all the files under specified directory to SE and DFC

Usage:
  ihepdirac_dms_add_dir [option|cfgfile] DFCDir LocalDir SE

Example:
  ihepdirac_dms_add_dir /juno/user/z/zhangxm/9743 /junofs/grid/user/z/zhangxm/9743 IHEP-JUNOEOS

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import os
import sys

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit

@DIRACScript()
def main():

    Script.parseCommandLine(ignoreErrors = False)
    args = Script.getPositionalArgs()

    if len(args) != 3:
        Script.showHelp()
        exit(1)

    dfcDir = args[0]
    localDir = args[1]
    toSE = args[2]

    from DIRAC.Interfaces.API.Dirac import Dirac
    dirac = Dirac()

    lfnList = []
    pfnList = []

    for root, dirs, files in os.walk(localDir):
        if not root.startswith(localDir):
            gLogger.error('Can not find corrent lfn')
            exit(1)
        relRoot = root[len(localDir):].lstrip('/')
        for f in files:
            fullFn = os.path.join(root, f)
            lfn = os.path.join(dfcDir, relRoot, f)
            lfnList.append(lfn)
            pfnList.append(fullFn)

    gLogger.notice('%s files will be added to DFC "%s"' % (len(lfnList), dfcDir))

    for lfn, pfn in zip(lfnList, pfnList):
        gLogger.debug('Add file to DFC: %s' % lfn)
        result = dirac.addFile(lfn, pfn, toSE)
        if not result['OK']:
            gLogger.error('Can not add file to DFC "%s": %s' % (lfn, result['Message']))
            exit(1)
        gLogger.debug('File upload successfully: %s' % f)
    gLogger.notice('%s files added to DFC' % len(lfnList))

if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Create JUNO production

Usage:
   ihepdirac-juno-make-reconstruction [option|cfgfile] [process]

Example:
   $ihepdirac-juno-make-reconstruction --example > rec.ini
   $ihepdirac-juno-make-reconstruction --ini myrec.ini
   $ihepdirac-juno-make-reconstruction ChainIBD
   $ihepdirac-juno-make-reconstruction --ini myrec.ini --dryrun
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit
from DIRAC.Core.Base import Script

import os
import sys
import re
import configparser
import json

Script.registerSwitch('i:', 'ini=', 'Ini file, default to "prod.ini"')
Script.registerSwitch(
    'r', 'dryrun', 'Only parse the configuration, do not submit transformation')
Script.registerSwitch('e', 'example', 'Display prod.ini example')
Script.parseCommandLine(ignoreErrors=False)

from DIRAC.ConfigurationSystem.Client.Helpers import Registry
from DIRAC.Core.Security.ProxyInfo import getProxyInfo
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

from DIRAC.Interfaces.API.Job import Job
from DIRAC.Interfaces.API.Dirac import Dirac

from DIRAC.Core.Workflow.Parameter import Parameter

DEBUG = True

URL_ROOT = 'http://dirac-code.ihep.ac.cn/juno/ts'

PROD_EXAMPLE = '''\
; Parameters could be put in the [all] section, or the process specified section
; The parameters in the specified section will overwrite those in the [all] section

; Common parameters
[all]
; Dryrun mode will only test the configuration and do not really submit any jobs
; This can also be passed in the command argument
;dryrun = true

; The process name "Chain" indicates that more parameters are selected from the [Chain] section
; This can also be passed in the command argument
process = Chain

; JUNO software version
softwareVersion = J17v1r1

prodName = JUNORecTest

; The prodNameSuffix will be added to the prodName in order to solve the transformation name confliction
;prodNameSuffix = _new

; The transformation group name
transGroup = JUNO_rec_test

; If outputType is "reconstruction", the root directory will be /juno/reconstruction
; If outputType is "user" or something else, the root directory will be under your user directory /juno/user/x/xxx
;outputType = reconstruction 

; The sub directory relative to the root directory
outputSubDir = zhangxm/test001

;outputSE = IHEP-STORM

; "closest" means upload the job output data to the closest SE
; If no closest SE is found for this site, then upload to the SE defined by "outputSE"
outputMode = closest

; If no site specified, all available sites will be chosen
;site = GRID.INFN-CNAF.it CLOUD.JINRONE.ru GRID.IN2P3.fr
;bannedsite = GRID.IHEP.cn

moveFlavor = Replication
;moveFlavor = Moving

; How many files to move in a single request
;moveGroupSize = 10

;moveSourceSE = IHEP-STORM CNAF-STORM JINR-JUNO IN2P3-DCACHE
;moveSourceSE = CNAF-STORM JINR-JUNO IN2P3-DCACHE
;moveTargetSE = IHEP-STORM CNAF-STORM
moveTargetSE = IHEP-STORM
movePlugin = Broadcast
;movePlugin = Standard

; The parameters in this section will overwrite what's in [all]
[Chain]
inputQuery = "transID=252"

; The root directory will be /<outputType dir>/<outputSubDir>/<softwareVersion>/<workDir>
workDir = Rawdata 

; workflow = calib rec
workflow = calib rec
moveType = calib rec 

;cal-mode = 
;rec-mode = 

'''


def _setMetaData(directory, meta):
    fcc = FileCatalogClient()
    res = fcc.createDirectory(directory)
    if not res['OK']:
        raise Exception('DFC createDirectory for "{0}" error: {1}'.format(
            directory, res['Message']))
    res = fcc.setMetadata(directory, meta)
    if not res['OK']:
        raise Exception('DFC setMetadata for "{0}" error: {1}'.format(
            directory, res['Message']))


def _getMetaData(directory):
    fcc = FileCatalogClient()
    res = fcc.getDirectoryUserMetadata(directory)
    if not res['OK']:
        return {}
    return res['Value']

def _checkProxy():
    """checks if the proxy has the ProductionManagement property and belongs to a VO"""
    proxyInfo = getProxyInfo()
    gLogger.debug("DEBUG: the proxyInfo used now: %s\n", proxyInfo)
    if not proxyInfo['OK']:
      gLogger.error("ERROR: No Proxy present")
      return False
    proxyValues = proxyInfo.get('Value', {})
    group = proxyValues.get('group', '')

    groupProperties = proxyValues.get('groupProperties', [])

    if groupProperties:
      if 'ProductionManagement' not in groupProperties:
        gLogger.error("ERROR: Not allowed to create production, you need a ProductionManagement proxy.")
        return False
    else:
      gLogger.error("ERROR: Could not determine Proxy properties, you do not have the right proxy.")
      return False
    return True

class Param(object):
    def __init__(self, configFile='', paramCmd={}):
        self.__configFile = configFile
        self.__paramCmd = paramCmd

        self.__param = {}
        self.__loadParam()
        self.__processParam()

        gLogger.debug('Final param: {0}'.format(self.__param))

    def __loadParam(self):
        if not self.__configFile:
            return
        config = configrparser.ConfigParser()
        config.optionxform = str
        config.read(self.__configFile)
        self.__param.update(dict(config.items('all')))
        self.__param.update(self.__paramCmd)
        if 'process' in self.__param:
            self.__param.update(dict(config.items(self.__param['process'])))
        self.__param.update(self.__paramCmd)    # Update paramCmd again to overwrite ini

    def __processParam(self):
        def parseList(s):
            return s.strip().split()

        def parseBool(s):
            if s.lower() in ['true', 'yes']:
                return True
            return False

        for key in ['softwareVersion', 'process']:
            if key not in self.__param or not self.__param[key]:
                raise Exception('Param "{0}" must be specified'.format(key))

        self.__param.setdefault('prodName', 'JUNOProd')
        self.__param.setdefault('transGroup', 'JUNO-Prod')
        self.__param.setdefault('outputType', 'user')
        self.__param.setdefault('outputSE', 'IHEP-STORM')
        self.__param.setdefault('outputMode', 'closest')
        self.__param.setdefault('seed', '0')
        self.__param.setdefault('workDir', self.__param['process'])
        self.__param.setdefault('moveFlavor', 'Replication')
        self.__param.setdefault('movePlugin', 'Broadcast')

        self.__param['numberOfTasks'] = int(self.__param.get('njobs', '1'))
        self.__param['evtmax'] = int(self.__param.get('evtmax', '1'))
        self.__param['max2dir'] = int(self.__param.get('max2dir', '10000'))
        self.__param['moveGroupSize'] = int(
            self.__param.get('moveGroupSize', '10'))

        self.__param['site'] = parseList(self.__param.get('site', ''))
        self.__param['bannedsite'] = parseList(self.__param.get('bannedsite', ''))
        self.__param['workflow'] = parseList(self.__param.get('workflow', ''))
        self.__param['moveType'] = parseList(self.__param.get('moveType', ''))
        self.__param['moveSourceSE'] = parseList(
            self.__param.get('moveSourceSE', 'IHEP-STORM'))
        self.__param['moveTargetSE'] = parseList(
            self.__param.get('moveTargetSE', 'IHEP-STORM'))

        self.__param['dryrun'] = parseBool(self.__param.get('dryrun', 'false'))
        
        self.__param['inputQuery'] = self.__param.get('inputQuery', '')
    # TODO: change inputQuery into inputMeta, a dictionary type
    

    @property
    def param(self):
        return self.__param


class ProdMove(object):
    def __init__(self, transType, transGroup, transName='unknown', flavour='Replication', description='Production move',
                 plugin='Broadcast', inputMeta={}, sourceSE=[], targetSE='IHEP-STORM', groupSize=1):
        self.__transType = transType
        self.__transGroup = transGroup
        self.__transName = transName
        self.__flavour = flavour
        self.__description = description
        self.__plugin = plugin
        self.__inputMeta = inputMeta
        self.__sourceSE = sourceSE
        self.__targetSE = targetSE
        self.__groupSize = groupSize

    def createTransformation(self):
        ########################################
        # Transformation definition
        ########################################
        t = Transformation()

        t.setTransformationName(self.__transName)
        t.setType(self.__transType)
        t.setDescription(self.__description)
        t.setLongDescription(self.__description)
        t.setGroupSize(self.__groupSize)
        if self.__transGroup:
            t.setTransformationGroup(self.__transGroup)
        t.setPlugin(self.__plugin)

#        t.setSourceSE(self.__sourceSE)
        t.setTargetSE(self.__targetSE)

        transBody = '' 

#        transBody.append(
#            ("ReplicateAndRegister", {"TargetSE": ','.join(self.__targetSE)}))

#        for tse in self.__targetSE:
#            sse = list(set(self.__sourceSE) - set([tse]))
#            transBody.append(("ReplicateAndRegister", {"SourceSE": ','.join(sse), "TargetSE": ','.join(tse)}))
#
#        if self.__flavour == 'Moving':
#            for sse in self.__sourceSE:
#                if sse in self.__targetSE:
#                    continue
#                gLogger.debug('Remove from SE: {0}'.format(sse))
#                transBody.append(("RemoveReplica", {"TargetSE": ','.join(sse)}))
#
#        transBody.append(("ReplicateAndRegister", {"SourceSE": ','.join(
#            self.__sourceSE), "TargetSE": ','.join(self.__targetSE)}))
#        if self.__flavour == 'Moving':
#            transBody.append(
#                ("RemoveReplica", {"TargetSE": ','.join(self.__sourceSE)}))

        t.setBody(transBody)

        ########################################
        # Transformation submission
        ########################################
        res = t.addTransformation()
        if not res['OK']:
            raise Exception(
                'Add transformation error: {0}'.format(res['Message']))

        t.setStatus("Active")
        t.setAgentType("Automatic")

        currtrans = t.getTransformationID()['Value']

        if self.__inputMeta:
            client = TransformationClient()
            res = client.createTransformationMetaQuery(
                currtrans, self.__inputMeta, 'Input')
            if not res['OK']:
                raise Exception(
                    'Create transformation query error: {0}'.format(res['Message']))

        return str(currtrans)


class ProdStep(object):
    def __init__(self, executable, transType, transGroup, softwareVersion,
                 application, stepName='unknown', description='Reconstruction step',
                 inputMeta={}, extraArgs='', inputData=None,
                 outputPath='/juno/test/prod', outputSE='IHEP-STORM', outputPattern='*.root',
                 site=None, bannedsite=None, outputMode='closest', maxNumberOfTasks=1):
        self.__executable = executable
        self.__transType = transType
        self.__transGroup = transGroup
        self.__softwareVersion = softwareVersion
        self.__application = application
        self.__stepName = stepName
        self.__description = description
        self.__inputMeta = inputMeta
        self.__extraArgs = extraArgs
        self.__inputData = inputData
        self.__outputPath = outputPath
        self.__outputSE = outputSE
        self.__outputPattern = outputPattern
        self.__site = site
        self.__bannedsite = bannedsite
        self.__outputMode = outputMode
        self.__maxNumberOfTasks = maxNumberOfTasks

        self.__job = None

    def createJob(self):
        job = Job()
        job.setName(self.__stepName)
        job.setOutputSandbox(['*log'])

        job.setExecutable(
            '/usr/bin/wget', arguments='"{0}/{1}"'.format(URL_ROOT, self.__executable))
        job.setExecutable(
            '/bin/chmod', arguments='+x "{0}"'.format(self.__executable))

        arguments = '"{0}" "{1}" "{2}" "{3}" "{4}" "{5}" @{{JOB_ID}}'.format(
            self.__softwareVersion, self.__application, self.__outputPath,
            self.__outputPattern, self.__outputSE, self.__outputMode)
        if self.__extraArgs:
            arguments += ' ' + self.__extraArgs
        job.setExecutable(self.__executable, arguments=arguments)

        # failover for failed jobs
        job.setExecutable('/bin/ls -l', modulesList=['Script', 'FailoverRequest'])

        if self.__inputData:
            job.setInputData(self.__inputData)

        if self.__site:
            job.setDestination(self.__site)

        if self.__bannedsite:
            job.setBannedSites(self.__bannedsite)

        job.setOutputSandbox(['app.out','app.err','Script3_CodeOutput.log'])

        self.__job = job

    def submitJob(self):
        dirac = Dirac()
        res = dirac.submitJob(self.__job)
        gLogger.notice('Job submitted: {0}'.format(res["Value"]))
        return res

    def createTransformation(self):
        ########################################
        # Transformation definition
        ########################################
        t = Transformation()

        t.setTransformationName(self.__stepName)
        t.setType(self.__transType)
        t.setDescription(self.__description)
        t.setLongDescription(self.__description)
        t.setGroupSize(1)
        if self.__transGroup:
            t.setTransformationGroup(self.__transGroup)
        # set the job workflow to the transformation
        t.setBody(self.__job.workflow.toXML())

        ########################################
        # Transformation submission
        ########################################
        res = t.addTransformation()
        if not res['OK']:
            raise Exception(
                'Add transformation error: {0}'.format(res['Message']))

        t.setStatus("Active")
        t.setAgentType("Automatic")

        currtrans = t.getTransformationID()['Value']

        if self.__inputMeta:
            client = TransformationClient()
            res = client.createTransformationInputDataQuery(
                currtrans, self.__inputMeta)
            if not res['OK']:
                raise Exception(
                    'Create transformation query error: {0}'.format(res['Message']))

        return str(currtrans)


class ProdChain(object):
    def __init__(self, param):
        self.__param = param
        self.__transIDs = {}

        self.__prodPrefix = '{0}{1}-{2}-{3}'.format(param['prodName'], param.get(
            'prodNameSuffix', ''), param['softwareVersion'], param['workDir'])

        self.__ownerAndGroup()

        outputSubDir = self.__param['outputSubDir'].strip('/')
        if self.__param['outputType'] == 'reconstruction':
            self.__outputRoot = os.path.join(self.__prodHome, outputSubDir)
        else:
            self.__outputRoot = os.path.join(self.__userHome, outputSubDir)

        self.__prepareDir()

        gLogger.notice('Owner: {0}'.format(self.__owner))
        gLogger.notice('OwnerGroup: {0}'.format(self.__ownerGroup))
        gLogger.notice('VO: {0}'.format(self.__vo))
        gLogger.notice('OutputRoot: {0}'.format(self.__outputRoot))
        gLogger.notice('ProdRoot: {0}'.format(self.__prodRoot))
        gLogger.notice('ProdPrefix: {0}'.format(self.__prodPrefix))

    def __ownerAndGroup(self):
        res = getProxyInfo(False, False)
        if not res['OK']:
            raise Exception('GetProxyInfo error: {0}'.format(res['Message']))
        self.__owner = res['Value']['username']
        self.__ownerGroup = res['Value']['group']
        self.__vo = Registry.getVOMSVOForGroup(self.__ownerGroup)
        self.__voHome = '/{0}'.format(self.__vo)
        self.__prodHome = '/{0}/reconstruction'.format(self.__vo)
        self.__userHome = '/{0}/user/{1:.1}/{1}'.format(
            self.__vo, self.__owner)

    def __prepareDir(self):
        outputPath = self.__outputRoot
        for d in ['softwareVersion', 'workDir']:
            if d == 'workDir':
                key = 'process'
            else:
                key = d
            outputPath = os.path.join(outputPath, self.__param[d])
            _setMetaData(outputPath, {key: self.__param[key]})

        self.__prodRoot = outputPath

    def __getOutputPath(self, application):
        #print("application path", os.path.join(self.__prodRoot, application))
        return os.path.join(self.__prodRoot, application)

    def __getTransID(self, application):
        meta = _getMetaData(self.__getOutputPath(application))
        if 'transID' not in meta:
            return ''
        return meta['transID']

    def __getMeta(self, application):
        if not application:
            return {}

        meta = {}
        meta['softwareVersion'] = self.__param['softwareVersion']
        meta['process'] = self.__param['process']
        meta['application'] = application

        transID = self.__getTransID(application)
        if transID:
            meta['transID'] = transID

        return meta

    def __setMeta(self, application):
        outputPath = self.__prodRoot

        meta = {}
        meta['application'] = application
        if application in self.__transIDs:
            meta['transID'] = self.__transIDs[application]
        outputPath = os.path.join(outputPath, application)
        print("set meta data:", outputPath, meta)
        _setMetaData(outputPath, meta)

    def createStep(self, application, transType, prevApp=None, inputMeta={}):
        if prevApp:
            inputMeta = self.__getMeta(prevApp)
            if 'transID' not in inputMeta:
                gLogger.error('{0}: Transformation not found for previous application "{1}"'.format(
                    application, prevApp))
                return
            gLogger.notice('{0}: Input transformation "{1}" from "{2}"'.format(
                application, inputMeta['transID'], prevApp))

        step_mode = self.__param.get(application + '-mode', '')
        if step_mode:
            gLogger.notice('{0}-mode: {1}'.format(application, step_mode))

        extraArgs = '{0} {1} "{2}" {3}'.format(
            self.__param['evtmax'], self.__param['seed'], step_mode, self.__param['max2dir'])
        stepArg = dict(
            executable='bootstrap.sh',
            transType=transType,
            transGroup=self.__param.get('transGroup'),
            softwareVersion=self.__param['softwareVersion'],
            application=application,
            stepName='{0}-{1}'.format(self.__prodPrefix, application),
            description='{0} for {1}'.format(
                application, self.__param['process']),
            extraArgs=extraArgs,
            inputMeta=inputMeta,
            outputPath=self.__getOutputPath(application),
            outputSE=self.__param['outputSE'],
            outputPattern='{0}*-*.root'.format(application),
            site=self.__param.get('site'),
            bannedsite=self.__param.get('bannedsite'),
            outputMode=self.__param['outputMode'],
            maxNumberOfTasks=self.__param['numberOfTasks'],
        )

        print("inputMeta:", inputMeta)

        gLogger.notice('{0}: Create transformation...'.format(application))

        if self.__param['dryrun']:
            transID = 'dryrun'
        else:
            prodStep = ProdStep(**stepArg)
            prodStep.createJob()
            transID = prodStep.createTransformation()

        #self.__transIDs.setdefault({})
        self.__transIDs[application] = transID

        if not self.__param['dryrun']:
            self.__setMeta(application)

    def createMove(self, application, transType):
        inputMeta = self.__getMeta(application)
        if 'transID' not in inputMeta:
            gLogger.error(
                '{0}-move: Transformation not found for application "{0}"'.format(application))
            return
        gLogger.notice(
            '{0}-move: Input transformation "{1}" from "{0}"'.format(application, inputMeta['transID']))

        moveArg = dict(
            transType=transType,
            transGroup=self.__param.get('transGroup'),
            transName='{0}-{1}-{2}'.format(self.__prodPrefix,
                                               application, self.__param.get('moveFlavor')),
            flavour=self.__param.get('moveFlavor'),
            description='Move {0} for {1}'.format(
                application, self.__param['process']),
            plugin=self.__param['movePlugin'],
            inputMeta=inputMeta,
            sourceSE=self.__param['moveSourceSE'],
            targetSE=self.__param['moveTargetSE'],
            groupSize=self.__param['moveGroupSize'],
        )

        gLogger.notice(
            '{0}-move: Create transformation...'.format(application))

        if self.__param['dryrun']:
            transID = 'dryrun'
        else:
            prodMove = ProdMove(**moveArg)
            transID = prodMove.createTransformation()

    def createAllTransformations(self):
  
        inputMeta = json.loads(self.__param['inputQuery'])
        gLogger.notice('\nInput metadata: {0}'.format(inputMeta))

        for step in ['elecsim','calib', 'rec', 'rawrec']:
            if step not in self.__param['workflow']:
                continue

            if step == 'elecsim':
                self.createStep('elecsim','ElecSimulation-JUNO',None, inputMeta)
            if step == 'calib':
                self.createStep('calib','Calibration-JUNO', 'elecsim')
            if step == 'rec':
                self.createStep('rec','DataReconstruction-JUNO', 'calib')
            if step == 'rawrec':
                self.createStep('calib', 'Calibration-JUNO', None, inputMeta)
                self.createStep('rec', 'DataReconstruction-JUNO', 'calib')

        for step in ['elecsim','calib','rec', 'rawrec']:
            if step not in self.__param['moveType']:
                continue
            gLogger.notice('createMove-step:{0}'.format(step))
            self.createMove(step, 'Replication-JUNO')

@DIRACScript()
def main():

    if not _checkProxy():
       gLogger.error('ERROR: You don\'t have proper proxy to use prodSys!')
       return 1

    args = Script.getPositionalArgs()
    switches = Script.getUnprocessedSwitches()

    configFile = 'rec.ini'
    paramCmd = {}
    displayExample = False

    for k, v in switches:
        if k == 'i' or k == 'ini':
            configFile = v
        if k == 'r' or k == 'dryrun':
            paramCmd['dryrun'] = 'true'
        if k == 'e' or k == 'example':
            displayExample = True

    if displayExample:
        sys.stdout.write(PROD_EXAMPLE)
        return 0

    if args:
        paramCmd['process'] = args[0]

    par = Param(configFile, paramCmd)

    chain = ProdChain(par.param)
    chain.createAllTransformations()

    return 0


if __name__ == '__main__':
    try:
        exit(main())
    except Exception as e:
        if DEBUG:
            raise
        gLogger.error('{0}'.format(e))
        exit(1)



""" TestBase

  Base class for all tests.

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from future import standard_library
standard_library.install_aliases()
from builtins import *
from builtins import object
import threading
from abc   import ABCMeta,abstractmethod
from future.utils import with_metaclass


__RCSID__ = '$Id:  $'

LOCK = threading.Lock()


class TestBase( with_metaclass(ABCMeta, object) ):
  """
    TestBase is a simple base class for all tests. Real test classes should
    implement its doTest and getTestResult method.
  """

  def __init__( self, args = None, apis = None ):
    self.apis = apis or {}
    self.args = args or {}


  @abstractmethod
  def doTest( self, elementDict ):
    """
      to be extended by real tests.
    """

    return None



#!/usr/bin/env python
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import json

import DIRAC
from DIRAC import S_OK, S_ERROR

from DIRAC.Core.Base import Script

Script.setUsageMessage( """
Reschedule jobs in the task. Only reschedule failed jobs by default

Usage:
   %s [option] ... [TaskID] ...
""" % Script.scriptName )

Script.registerSwitch( "a",  "all",        "Reschdule all jobs in the task" )

Script.parseCommandLine( ignoreErrors = False )
options = Script.getUnprocessedSwitches()
args = Script.getPositionalArgs()

from IHEPDIRAC.WorkloadManagementSystem.Client.TaskClient   import TaskClient
taskClient = TaskClient()

def rescheduleTask(taskID, status=[]):
  result = taskClient.rescheduleTask(taskID, status)
  if not result['OK']:
    print('Reschedule task error: %s' % result['Message'])
    return
  print('Task %s rescheduled' % taskID)

def main():
  if len(args) < 1:
    Script.showHelp()
    return

  status = ['Failed']
  for option in options:
    (switch, val) = option
    if switch == 'a' or switch == 'all':
      status = []

  for taskID in args:
    taskID = int(taskID)
    rescheduleTask(taskID, status)
    print('')

if __name__ == '__main__':
  main()



#!/usr/bin/env python
"""
Start transfer according to DFC query with transformation system

Usage:
  ihepdirac-transformation-transfer-metadata [option|cfgfile] TransformationName MetaTransfer TargetSE

Example:
  $ihepdirac-transformation-transfer-metadata Meassurements_DAQ_CNAF juno_transfer=Pmt/container/Meassurements CNAF-STORM
  $ihepdirac-transformation-transfer-metadata Meassurements_DAQ_IN2P3 -p Standard juno_transfer=Pmt/container/Meassurements IN2P3-DCACHE
  $ihepdirac-transformation-transfer-metadata Meassurements_DAQ_JINR -t Transfer-JUNO juno_transfer=Pmt/container/Meassurements -f IHEP-STORM JINR-JUNO

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit

from DIRAC.DataManagementSystem.Client.MetaQuery import MetaQuery, FILE_STANDARD_METAKEYS

from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient


@DIRACScript()
def main():

    Script.registerSwitch( 't:', 'transformationType=', 'Specify transformation type')
    Script.registerSwitch( 'g:', 'groupSize=', 'Group size for each task')
    Script.registerSwitch( 'p:', 'plugin=', 'Plugin, default to "Broadcast"')
    Script.registerSwitch( 'f:', 'sourceSE=', 'SourceSE')
    Script.parseCommandLine(ignoreErrors = False)
    args = Script.getPositionalArgs()

    if len(args) != 3:
        Script.showHelp()
        exit(1)

    transformationName = args[0]
    metaTransfer = args[1]
    targetSE = args[2]

    sourceSE = ''
    groupSize = 100
    transformationType = 'Transfer-JUNO'
    plugin = 'Broadcast'

    switches = Script.getUnprocessedSwitches()
    for switch in switches:
        if switch[0] == 'g' or switch[0] == 'groupSize':
            groupSize = int(switch[1])
        if switch[0] == 't' or switch[0] == 'transformationType':
            transformationType = switch[1]
        if switch[0] == 'p' or switch[0] == 'plugin':
            plugin = switch[1]
        if switch[0] == 'f' or switch[0] == 'sourceSE':
            sourceSE = switch[1]

    from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

    fcc = FileCatalogClient('DataManagement/FileCatalog')

    result = fcc.getMetadataFields()
    if not result['OK']:
        gLogger.error("Error: %s" % result['Message'])
        exit(0)
    if not result['Value']:
        gLogger.error("Error: no metadata fields defined")
        exit(0)
    typeDict = result['Value']['FileMetaFields']
    typeDict.update(result['Value']['DirectoryMetaFields'])
    # Special meta tags
    typeDict.update(FILE_STANDARD_METAKEYS)
    mq = MetaQuery(typeDict=typeDict)
    mq.setMetaQuery([metaTransfer])
    query = mq.getMetaQuery()
    gLogger.notice('Query: {0}'.format(query))

    t = Transformation( )
    tc = TransformationClient( )
    t.setTransformationName(transformationName) # Must be unique
    t.setTransformationGroup("Transfer")
    t.setType(transformationType)
    t.setPlugin(plugin)
    t.setDescription("Data Transfer")
    t.setLongDescription("Data Transfer") # Mandatory
    t.setGroupSize(groupSize) # Here you specify how many files should be grouped within he same request, e.g. 100

    transBody = ''
    t.setBody(transBody)

    if sourceSE:
        res = t.setSourceSE(sourceSE)
        if not res['OK']:
            gLogger.error("SourceSE not valid: %s" % res['Message'])
            exit(1)

    res = t.setTargetSE(targetSE)
    if not res['OK']:
        gLogger.error("TargetSE not valid: %s" % res['Message'])
        exit(1)

    result = t.addTransformation() # Transformation is created here
    if not result['OK']:
        gLogger.error('Can not add transformation: %s' % result['Message'])
        exit(2)

    t.setStatus("Active")
    t.setAgentType("Automatic")
    transID = t.getTransformationID()

    result = tc.createTransformationMetaQuery(transID['Value'], query, 'Input')
    if not result['OK']:
        gLogger.error('Can not create query to transformation: %s' % result['Message'])
        exit(2)

if __name__ == "__main__":
    main()



#! /usr/bin/env python
"""
Remove specified replica from SE under specified directory
Usage:
  ihepdirac_dms_rm_dir_replicas [option|cfgfile] DFCDir SE

Example:
  $ ihepdirac_dms_rm_dir_replicas /juno/production/muon/prd100 IHEP-STORM
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import os
import sys

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit


from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
fcc = FileCatalogClient('DataManagement/FileCatalog')

from DIRAC.DataManagementSystem.Client.DataManager import DataManager
dm = DataManager()

counterFile = 0
counterDir = 0


def removeFromDir(d, SE):
    global counterFile
    global counterDir

    result = fcc.listDirectory(d)
    if not result['OK']:
        gLogger.error('Failed to list directory %s: %s' %
                      (d, result['Message']))
        return

    gLogger.notice('Removing replicas from dir: %s' % d)

    if result['Value']['Successful'][d]['Files']:
        files = result['Value']['Successful'][d]['Files']
        fileNumber = len(files)
        gLogger.notice(
            'Removing replicas of {0} files from dir "{1}"'.format(fileNumber, d))
        counterFile += fileNumber
        dm.removeReplica(SE, files)

    if result['Value']['Successful'][d]['SubDirs']:
        for subdir in result['Value']['Successful'][d]['SubDirs']:
            removeFromDir(subdir, SE)

    counterDir += 1


@DIRACScript()
def main():

    Script.parseCommandLine(ignoreErrors=False)

    args = Script.getPositionalArgs()

    if len(args) != 2:
        Script.showHelp()
        exit(1)

    dfcDir = args[0]
    SE = args[1]

    removeFromDir(dfcDir, SE)

    gLogger.notice('%s directories and %s files deleted' %
               (counterDir, counterFile))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import json

import DIRAC
from DIRAC import S_OK, S_ERROR

from DIRAC.Core.Base import Script

Script.setUsageMessage( """
Show job information

Usage:
   %s [option] ... [JobID] ...
""" % Script.scriptName )

Script.parseCommandLine( ignoreErrors = False )
options = Script.getUnprocessedSwitches()
args = Script.getPositionalArgs()

from IHEPDIRAC.WorkloadManagementSystem.Client.TaskClient   import TaskClient
taskClient = TaskClient()

def showPairs(pairs):
  width = 0
  for pair in pairs:
    width = max(width, len(pair[0]))
  format = '- %%-%ds : %%s' % width
  for k,v in pairs:
    print(format % (k, v))

def showJobs(jobIDs):
  outFields = ['TaskID', 'JobID', 'Info']
  result = taskClient.getJobs(jobIDs, outFields)
  if not result['OK']:
    print('Show jobs error: %s' % result['Message'])
    return

  if not result['Value']:
    print('No task found for jobs: %s' % jobIDs)
    return

  for v in result['Value']:
    taskID = v[0]
    jobID = v[1]
    jobInfo = json.loads(v[2])

    pairs = []
    pairs.append(['TaskID', str(taskID)])
    for k,v in sorted(jobInfo.iteritems(), key=lambda d:d[0]):
      if type(v) == type([]):
        v = ', '.join(v)
      pairs.append([k, v])

    print('JobID : %s' % jobID)
    showPairs(pairs)

    print('')

def main():
  if len(args) < 1:
    Script.showHelp()
    return

  jobIDs = []
  for jobID in args:
    jobIDs.append(int(jobID))

  showJobs(jobIDs)

  print('Totally %s job(s) displayed' % len(jobIDs))

if __name__ == '__main__':
  main()



#! /usr/bin/env python
"""
Scan dark files in assigned SEs.
Usage:
  ihepdirac-dms-scan-unregistered [option|cfgfile] DFCDir SE

Example:
  $ ihepdirac-dms-scan-unregistered /juno/production/muon/prd100 IHEP-STORM
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import os
import sys
import re
from stat import S_ISREG, S_ISDIR, S_IXUSR, S_IRUSR, S_IWUSR, S_IRWXG, S_IRWXU, S_IRWXO

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit

from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
fcc = FileCatalogClient('DataManagement/FileCatalog')

from DIRAC.DataManagementSystem.Client.DataManager import DataManager
dm = DataManager()

Script.registerSwitch( 'e', 'delete', 'Delete dark files from SE after scan')
Script.registerSwitch( 'r:', 'register=', 'Register dark files or unlinked files after scan, <value>=[dark|unlinked|both]')
Script.registerSwitch( 'l', 'list', 'Use a local file list as input')
Script.registerSwitch( 'k', 'skipsub', 'Skip sub-directory when scanning')
Script.registerSwitch( 'm', 'srmmode', 'Special registering mode for SRM when more than 2000 files in a directory, only scan unlinked file')
Script.parseCommandLine(ignoreErrors = False)

import gfal2
g=gfal2.creat_context()

from DIRAC.Core.Utilities.File import makeGuid

# input data
args = Script.getPositionalArgs()
if len(args) != 2:
    Script.showHelp()
    exit(1)

INPUT = args[0]
SITE = args[1]

deletionSwitch = False
registerOption = 'none'
inputSwitch = False
srmSwitch = False
skipSwitch = False
switches = Script.getUnprocessedSwitches()
for switch in switches:
    if switch[0] == 'e' or switch[0] == 'delete':
        deletionSwitch = True
    if switch[0] == 'r' or switch[0] == 'register':
        registerOption = switch[1].lower()
    if switch[0] == 'l' or switch[0] == 'list':
        inputSwitch = True
    if switch[0] == 'm' or switch[0] == 'srmmode':
        srmSwitch = True
    if switch[0] == 'k' or switch[0] == 'skipsub':
        skipSwitch = True

if registerOption not in ['none', 'dark', 'unlinked', 'both']:
    gLogger.error("Error: '-r' option can only be 'dark', 'unlinked' or 'both'")
    exit(1)
if registerOption in ['both', 'dark'] and deletionSwitch:
    gLogger.error("Error: '-r dark/both' and '-e' cannot be used at the same time, please use '-r dark' when deleting")
    exit(1)
if registerOption in ['both', 'dark'] and srmSwitch:
    gLogger.error("Error: '-r dark/both' and '-m' cannot be used at the same time, please use '-r unlinked' when using srmmode")
    exit(1)

from DIRAC.Resources.Storage.StorageElement import StorageElement
se = StorageElement(SITE)

# initial list
protocols = ['root', 'http', 'gsiftp', 'srm']
fileList = []
fileDFCList = []
fileDarkList = []
fileUnlinkedList = []

inputFileList = []
if inputSwitch:
    inputFile = open(INPUT, 'r')
    tmpList = inputFile.readlines()
    for i in tmpList:
        inputFileList.append(i.rstrip('\n').rstrip('/'))
else:
    inputFileList.append(INPUT.rstrip('/'))
HEAD0 = se.getURL(inputFileList[0],protocols)['Value']['Successful'][inputFileList[0]][0:-len(inputFileList[0])]
HEAD = re.sub('\?', '\\?', HEAD0)
# print HEAD0

# special for SRM
if HEAD[0:6] == ("srm://"):
    gLogger.notice("Notice: This is a SRM SE. Cannot scan more than 2000 files! Please use srm mode if you need to scan and register more than 2000 files in a directory.\n")

def resolveUrl(dfcUrl):
    pfnUrl = se.getURL(dfcUrl, protocols)['Value']['Successful'][dfcUrl]
    return pfnUrl

# list all files recursively in a PFN directory
def listFile(pfnUrl):
    # skip "." and ".." when list file
    if pfnUrl[-2:] == ("/.") or pfnUrl[-3:] == ("/.."):
        return
    tmpList = g.listdir(pfnUrl)
    for i in tmpList:
        if isDir(pfnUrl + '/' + i):
            if not skipSwitch:
                listFile(pfnUrl + '/' + i)
            else:
                return
        else:
            fileList.append(pfnUrl + '/' + i)
            fileDFCList.append(re.sub(HEAD, '', pfnUrl + '/' + i))
    return

# compare a single file between DFC and PFN
def compareFile(dfcUrl):
    rstDist=dm.getReplicas(dfcUrl)
    if rstDist['OK'] == True:
        if dfcUrl in rstDist['Value']['Successful'].keys():
            if SITE in rstDist['Value']['Successful'][dfcUrl].keys():
                return True
            else:
                fileUnlinkedList.append(dfcUrl)
                return False
        else:
            fileDarkList.append(dfcUrl)
            return False
    else:
        gLogger.error("Dirac API Error!")
        exit(1)
    return False

# listFile func for srm mode
def srmListFile(dfcUrl):
    rstDist = fcc.listDirectory(dfcUrl)['Value']['Successful'][dfcUrl]
    if 'Files' in rstDist:
        tmpFileList = rstDist['Files'].keys()
        for i in tmpFileList:
            fileList.append(HEAD0 + '/' + i)
            fileDFCList.append(i)
    if 'SubDirs' in rstDist and not skipSwitch:
        tmpSubdirList = rstDist['SubDirs'].keys()
        for i in tmpSubdirList:
            srmListFile(i)
    return

# compare a single file between DFC and PFN, only scan unlinked files, cannot scan lost files
def srmCompareFile(dfcUrl):
    rstDist=dm.getReplicas(dfcUrl)
    if rstDist['OK']:
        if dfcUrl in rstDist['Value']['Successful'].keys():
            if SITE in rstDist['Value']['Successful'][dfcUrl].keys():
                return False
            try:
                g.stat(resolveUrl(dfcUrl))
            except gfal2.GError:
                return False
    gLogger.notice("Notice: Unlinked file found, %s" % dfcUrl)
    fileUnlinkedList.append(dfcUrl)
    return True

# judge whether a gfal return is a directory
def isDir(pfnUrl):
    try:
        rst = S_ISDIR(g.stat(pfnUrl).st_mode)
    except gfal2.GError:
        raise
    return S_ISDIR(g.stat(pfnUrl).st_mode)

# execute
@DIRACScript()
def main():
    for i in inputFileList:
        PFNURL = resolveUrl(i)

        # judge if file/path exist in SE. if not, move to next file/path
        try:
           isDir(PFNURL)
        except gfal2.GError:
           gLogger.notice("Warning: file or directory (%s) not found in assigned SE, skipped!\n" % PFNURL)
           continue

        if not isDir(PFNURL):
           fileList.append(PFNURL)
           fileDFCList.append(re.sub(HEAD, '', PFNURL))
        else:
           if not srmSwitch:
              listFile(PFNURL)
           else:
              srmListFile(i)

    # duplicate removal
    fileList_copy = list(set(fileList))
    fileList_copy.sort()
    fileDFCList_copy = list(set(fileDFCList))
    fileDFCList_copy.sort()

    # output
    if not srmSwitch:
       gLogger.debug("----------  Files found in assigned SE:  ----------")
    else:
       gLogger.debug("----------  Files found in assigned DFC:  ----------")
    for i in fileDFCList_copy:
       gLogger.debug(i)
       if not srmSwitch:
          compareFile(i)
       else:
          srmCompareFile(i)
    gLogger.notice("----------  Total: %d found files.  ----------\n" %len(fileDFCList_copy))

    if not srmSwitch:
       gLogger.debug("----------  Files unregistered (dark files):  ----------")
       for i in fileDarkList:
           gLogger.debug(i)
       gLogger.notice("----------  Total: %d dark files.  ----------\n" %len(fileDarkList))

    gLogger.debug("----------  Files registered but not register as assigned SE replicas:  ----------")
    for i in fileUnlinkedList:
        gLogger.debug(i)
    gLogger.notice("----------  Total: %d unlinked files.  ----------\n" %len(fileUnlinkedList))

    if deletionSwitch:
        for i in fileDarkList:
            g.unlink(HEAD0 + '/' + i)
            gLogger.debug("%s deleted!" %(HEAD0 + '/' + i))
        gLogger.notice("----------  All dark files deleted!  ----------\n")

    if registerOption != 'none':
        if registerOption in ['dark', 'both']:
            gLogger.debug("----------  Register dark files:  ----------")
            fileTupleBuffer = []
            for i in fileDarkList:
                fileStat = g.stat(HEAD0 + '/' + i)
                fileSize = fileStat.st_size
                fileGuid = makeGuid()
                fileSum = g.checksum(HEAD0 + '/' + i, "ADLER32")
                fileTuple = (i, HEAD0 + '/' + i, fileSize, SITE, fileGuid, fileSum)
                fileTupleBuffer.append(fileTuple)
                if len(fileTupleBuffer) > 499:
                    dm.registerFile(fileTupleBuffer)
                    fileTupleBuffer = []
                gLogger.debug(i + ' registered successfully!')
            if len(fileTupleBuffer) > 0:
                dm.registerFile(fileTupleBuffer)
        if registerOption in ['unlinked', 'both']:
            gLogger.debug("----------  Register unlinked files:  ----------")
            fileTupleBuffer = []
            for i in fileUnlinkedList:
                fileTuple = (i, HEAD0 + '/' + i, SITE)
                fileTupleBuffer.append(fileTuple)
                if len(fileTupleBuffer) > 499:
                    dm.registerReplica(fileTupleBuffer)
                    fileTupleBuffer = []
                gLogger.debug(i + ' registered successfully!')
            if len(fileTupleBuffer) > 0:
                dm.registerReplica(fileTupleBuffer)
        gLogger.notice("----------  Register finished  ----------\n")

if __name__ == "__main__":
    main()



""""

"""

def extension_metadata():
  import importlib.resources  # pylint: disable=import-error,no-name-in-module
  return {
      "primary_extension": True,
      "priority": 100,
      "setups": {
          "CAS_Production": "dips://prod-dirac.ihep.ac.cn:9135/Configuration/Server",
          "CAS_Production-cert": "dips://prod-dirac.ihep.ac.cn:9135/Configuration/Server",
      },
      "default_setup": "CAS_Production",
      "web_resources": {
          "static": [importlib.resources.files("IHEPDIRAC") / "WebApp" / "static"],  # pylint: disable=no-member
      },
  }



#!/usr/bin/env python

from DIRAC.Core.Base import Script
Script.parseCommandLine(ignoreErrors = False)
args = Script.getPositionalArgs()

import time
import tempfile

from DIRAC import S_OK, S_ERROR, gLogger, exit

from DIRAC.DataManagementSystem.Client.DataManager      import DataManager

lfn = args[0]
pfn = args[1]
se = args[2]

exit_code = 0
log = ''

dm = DataManager()

start = time.time()
result = dm.removeFile( lfn )
result = dm.putAndRegister( lfn, pfn, se )
uploadTime = time.time() - start
if result[ 'OK' ]:
  log += 'Succeed to upload file to SE %s.\n' % se
  log += 'Upload Time : %ss\n' % uploadTime

  start = time.time()
  result = dm.getReplica( lfn, se, tempfile.gettempdir() )
  downloadTime = time.time() - start
  if result[ 'OK' ]:
    log += 'Succeed to download file from SE %s.\n' % se
    log += 'Download Time : %ss\n' % downloadTime
  else:
    exit_code = 1
    log += 'Failed to download file from SE %s : %s\n' % ( se, result[ 'Message' ] )

  result = dm.removeFile( lfn )
  if result[ 'OK' ]:
    log += 'Succeed to delete file from SE %s.\n' % se
  else:
    log += 'Faile to delete file from SE %s : %s\n' % ( se, result[ 'Message' ] )

else:
  exit_code = 1
  log += 'Failed to upload file to SE %s : %s\n' % ( se, result[ 'Message' ] )

print(log)
exit(exit_code)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from dirac_webapp_packaging import extjs_cmdclass
from setuptools import setup
# This is required to allow editable pip installs while using the declarative configuration (setup.cfg)
setup(cmdclass=extjs_cmdclass)



#!/usr/bin/env python

import socket, subprocess, time

hostname = socket.gethostname()

commands = """
cd $TMPDIR
cvmfsdir="/cvmfs/boss.cern.ch"
if [ ! -d "/cvmfs/boss.cern.ch" ]; then
  cvmfsdir="/cvmfs/bes.ihep.ac.cn"
fi
source $cvmfsdir/slc5_amd64_gcc43/6.6.4.p01/setup.sh
source $cvmfsdir/slc5_amd64_gcc43/6.6.4.p01/scripts/6.6.4.p01/setup.sh
source $cvmfsdir/slc5_amd64_gcc43/6.6.4.p01/dist/6.6.4.p01/TestRelease/*/cmt/setup.sh
cp $cvmfsdir/slc5_amd64_gcc43/6.6.4.p01/dist/6.6.4.p01/TestRelease/*/run/rhopi.dec .
cp $cvmfsdir/slc5_amd64_gcc43/6.6.4.p01/dist/6.6.4.p01/TestRelease/*/run/jobOptions_sim.txt .
cp $cvmfsdir/slc5_amd64_gcc43/6.6.4.p01/dist/6.6.4.p01/TestRelease/*/run/jobOptions_rec.txt .

cat jobOptions_rec.txt | grep -v "BESEVENTMIXER" > jobOptions_rec1.txt
echo "DatabaseSvc.DbType=\\"sqlite\\";" >> jobOptions_sim.txt
echo "DatabaseSvc.SqliteDbPath=\\"$cvmfsdir/slc5_amd64_gcc43/6.6.4.p01/database/\\";" >>  jobOptions_sim.txt
echo "DatabaseSvc.DbType=\\"sqlite\\";" >> jobOptions_rec1.txt
echo "DatabaseSvc.SqliteDbPath=\\"$cvmfsdir/slc5_amd64_gcc43/6.6.4.p01/database/\\";" >>  jobOptions_rec1.txt
echo "MessageSvc.OutputLevel = 6;" >>  jobOptions_sim.txt
echo "MessageSvc.OutputLevel = 6;" >>  jobOptions_rec1.txt

time boss.exe jobOptions_sim.txt
time boss.exe jobOptions_rec1.txt

ls -latr .
"""

start = time.time()
subp = subprocess.Popen( commands, shell = True, stdout = subprocess.PIPE, stderr = subprocess.PIPE )
stdout, stderr = subp.communicate()
runningTime = time.time() - start

print('Host Name :', hostname)
print('Running Time :', runningTime)
print('\n')
if stdout:
    print('==============================Standard Output==============================\n')
    print(stdout)
    print('\n')
if stderr:
    print('==============================Standard Error===============================\n')
    print(stderr)



#!/usr/bin/env python
"""
Start transfer of all files under directory with transformation system
Usage:
 ihepdirac-transformation-transfer-dir [option|cfgfile] TransferName DFCDir SourceSE DestSE

Example: 
 $ihepdirac-transformation-transfer-dir transfer_jinr_ihep_2022 /juno/production/muon/prd010 IHEP-JUNOEOS JINR-EOS

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC import S_OK, S_ERROR, gLogger, exit

import sys

from DIRAC.TransformationSystem.Client.Transformation import Transformation
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

@DIRACScript()
def main():
    Script.parseCommandLine()
    args = Script.getPositionalArgs()
    if len(args) != 4:
        Script.showHelp()
        exit(1)

    transferName = args[0]
    inDir = args[1]
    fromSE = args[2]
    toSE = args[3]

    from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

    fcc = FileCatalogClient('DataManagement/FileCatalog')
    result = fcc.listDirectory(inDir)
    if not result['OK'] or 'Successful' not in result['Value'] or inDir not in result['Value']['Successful']:
       gLogger.error('Can not list directory %s' % inDir)
       exit(2)

    infileList = list(result['Value']['Successful'][inDir]['Files'].keys())
    sorted(infileList)

    print('%s files to transfer' % len(infileList))

    if not infileList:
        gLogger.Info('No file to transfer')
        exit(0)

    t = Transformation( )
    tc = TransformationClient( )
    t.setTransformationName(transferName) # Must be unique
    t.setTransformationGroup("Transfer")
    t.setType("Transfer-JUNO")
    #t.setPlugin("Standard") # Not needed. The default is 'Standard'
    t.setDescription("Test Data Transfer")
    t.setLongDescription( "Long description of Data Transfer" ) # Mandatory
    t.setGroupSize(3) # Here you specify how many files should be grouped within he same request, e.g. 100

    transBody = [ ( "ReplicateAndRegister", { "SourceSE": fromSE, "TargetSE": toSE }) ]

    t.setBody ( transBody ) # Mandatory

    result = t.addTransformation() # Transformation is created here
    if not result['OK']:
        gLogger.error('Can not add transformation: %s' % result['Message'])
        exit(2)

    t.setStatus("Active")
    t.setAgentType("Automatic")
    transID = t.getTransformationID()

    result = tc.addFilesToTransformation(transID['Value'], infileList) # Files are added here
    if not result['OK']:
        gLogger.error('Can not add files to transformation: %s' % result['Message'])
        exit(2)

    result = tc.setTransformationParameter( transID['Value'], 'Status', 'Flush' )
    if not result['OK']:
        gLogger.error('Can not flush transformation: %s' % result['Message'])
        exit(2)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
import json

import DIRAC
from DIRAC import S_OK, S_ERROR

from DIRAC.Core.Base import Script

Script.setUsageMessage( """
Get task progress of events

Usage:
   %s TaskID1 TaskID2 ...
""" % Script.scriptName )

Script.parseCommandLine( ignoreErrors = False )
options = Script.getUnprocessedSwitches()
args = Script.getPositionalArgs()

from IHEPDIRAC.WorkloadManagementSystem.Client.TaskClient   import TaskClient
taskClient = TaskClient()

from DIRAC.Core.DISET.RPCClient           import RPCClient
jobMonitor = RPCClient('WorkloadManagement/JobMonitoring')

def getTaskEventProgress(taskID):
    progress = {}

    result = taskClient.getTaskJobs(taskID)
    if not result['OK']:
        print(result['Message'])
        return progress
    jobIDs = result['Value']

    result = jobMonitor.getJobsStatus(jobIDs)
    if not result['OK']:
        print(result['Message'])
        return progress
    jobsStatus = result['Value']

    outFields = ['JobID', 'Info']
    result = taskClient.getJobs(jobIDs, outFields)
    if not result['OK']:
        print(result['Message'])
        return progress
    jobsInfo = {info[0] : json.loads(info[1]) for info in result['Value']}

    progress['Total'] = {'JobNum': 0, 'EventNum': 0}
    for jobID in jobIDs:
        if jobID in jobsStatus:
            status = jobsStatus[jobID]['Status']
        else:
            status = 'Deleted'

        if status not in progress:
            progress[status] = {'JobNum': 0, 'EventNum': 0}
        progress[status]['JobNum'] += 1
        progress['Total']['JobNum'] += 1

        if jobID in jobsInfo:
            evtNum = jobsInfo[jobID]['EventNum'] if 'EventNum' in jobsInfo[jobID] else 0
            progress[status]['EventNum'] += evtNum
            progress['Total']['EventNum'] += evtNum

    return progress

def printALine(progress, status):
    if status in progress:
        print('%-12s %12s %12s' % (status, progress[status]['JobNum'], progress[status]['EventNum']))

def main():
    if len(args) < 1:
        Script.showHelp()
        return

    fixedStatuses = ['Total', 'Done', 'Failed', 'Running', 'Waiting', 'Deleted']

    for taskID in args:
        taskID = int(taskID)
        progress = getTaskEventProgress(taskID)

        if not progress:
            continue

        print('='*80)
        print('Task %s progress:' % taskID)
        print('-'*12 + ' ' + '-'*12 + ' ' + '-'*12)
        print('%-12s %-12s %-12s' % ('Status', 'Job Number', 'Event Number'))
        print('-'*12 + ' ' + '-'*12 + ' ' + '-'*12)
        printALine(progress, 'Total')
        print('-'*12 + ' ' + '-'*12 + ' ' + '-'*12)
        for fixedStatus in fixedStatuses:
            if fixedStatus != 'Total':
                printALine(progress, fixedStatus)
        for p in progress:
            if p not in fixedStatuses:
                printALine(progress, p)
        print('-'*12 + ' ' + '-'*12 + ' ' + '-'*12)

        print('')

if __name__ == '__main__':
    main()
